------------------------------------------------------------------------------
--                                                                          --
--                            Matreshka Project                             --
--                                                                          --
--                               XML Processor                              --
--                                                                          --
--                        Runtime Library Component                         --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2010, Vadim Godunko <vgodunko@gmail.com>                     --
-- All rights reserved.                                                     --
--                                                                          --
-- Redistribution and use in source and binary forms, with or without       --
-- modification, are permitted provided that the following conditions       --
-- are met:                                                                 --
--                                                                          --
--  * Redistributions of source code must retain the above copyright        --
--    notice, this list of conditions and the following disclaimer.         --
--                                                                          --
--  * Redistributions in binary form must reproduce the above copyright     --
--    notice, this list of conditions and the following disclaimer in the   --
--    documentation and/or other materials provided with the distribution.  --
--                                                                          --
--  * Neither the name of the Vadim Godunko, IE nor the names of its        --
--    contributors may be used to endorse or promote products derived from  --
--    this software without specific prior written permission.              --
--                                                                          --
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS      --
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        --
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR    --
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     --
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   --
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED --
-- TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   --
-- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF   --
-- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING     --
-- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS       --
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.             --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 1040 $ $Date: 2010-09-05 10:15:40 +0400 (Sun, 05 Sep 2010) $
------------------------------------------------------------------------------
with Ada.Unchecked_Deallocation;

package body XML.SAX.Locators is

   ------------
   -- Adjust --
   ------------

   overriding procedure Adjust (Self : in out SAX_Locator) is
   begin
      Matreshka.Internals.Atomics.Counters.Increment
       (Self.Data.Counter'Access);
   end Adjust;

   ------------
   -- Column --
   ------------

   function Column (Self : SAX_Locator'Class) return Natural is
   begin
      return Self.Data.Column;
   end Column;

   --------------
   -- Encoding --
   --------------

   function Encoding
    (Self : SAX_Locator'Class) return League.Strings.Universal_String
   is
      pragma Unreferenced (Self);

   begin
      return League.Strings.Empty_Universal_String;
   end Encoding;

   --------------
   -- Finalize --
   --------------

   overriding procedure Finalize (Self : in out SAX_Locator) is
      procedure Free is
        new Ada.Unchecked_Deallocation (Shared_Locator, Shared_Locator_Access);

   begin
      --  Finalize can be called more than once (as specified by language
      --  standard), thus implementation should provide protection from
      --  multiple finalization.

      if Self.Data /= null then
         if Matreshka.Internals.Atomics.Counters.Decrement
             (Self.Data.Counter'Access)
         then
            Free (Self.Data);
         end if;
      end if;
   end Finalize;

   ----------
   -- Line --
   ----------

   function Line (Self : SAX_Locator'Class) return Natural is
   begin
      return Self.Data.Line;
   end Line;

   ---------------
   -- Public_Id --
   ---------------

   function Public_Id
    (Self : SAX_Locator'Class) return League.Strings.Universal_String
   is
      pragma Unreferenced (Self);

   begin
      return League.Strings.Empty_Universal_String;
   end Public_Id;

   ---------------
   -- System_Id --
   ---------------

   function System_Id
    (Self : SAX_Locator'Class) return League.Strings.Universal_String
   is
      pragma Unreferenced (Self);

   begin
      return League.Strings.Empty_Universal_String;
   end System_Id;

   -------------
   -- Version --
   -------------

   function Version
    (Self : SAX_Locator'Class) return League.Strings.Universal_String is
   begin
      if Self.Data.Version = 0 then
         return League.Strings.To_Universal_String ("1.0");

      elsif Self.Data.Version = 1 then
         return League.Strings.To_Universal_String ("1.1");

      else
         raise Program_Error;
      end if;
   end Version;

end XML.SAX.Locators;
